#
# spec file for package python-torch
#
# Copyright (c) 2025 SUSE LLC
#
# All modifications and additions to the file contributed by third parties
# remain the property of their copyright owners, unless otherwise agreed
# upon. The license for this file, and modifications and additions to the
# file, is the same license as for the pristine package itself (unless the
# license for the pristine package is not an Open Source License, in which
# case the license is the MIT License). An "Open Source License" is a
# license that conforms to the Open Source Definition (Version 1.9)
# published by the Open Source Initiative.

# Please submit bugfixes or comments via https://bugs.opensuse.org/
#


%define srcname pytorch
%define pname torch

%global flavor openmpi4%{nil}

%if "%{flavor}" == "standard"
%bcond_with cuda
%endif

%if "%{flavor}" == "cuda-10-2"
%bcond_without cuda
%define cudaver 10-2
%endif

%bcond_with mpi
%if "%{flavor}" == "openmpi4"
%bcond_without mpi
%bcond_without openmpi4
%global mpi_flavor openmpi
%define mpi_ext 4
%endif

%if "%{flavor}" == "vulkan"
%bcond_without vulkan
%global pkg_suffix -vulkan
%endif

%if %{with mpi}
%global pkg_suffix %{?mpi_flavor:-%{mpi_flavor}%{?mpi_ext}}
%define pkg_prefix %{_libdir}/mpi/gcc/%{mpi_flavor}%{?mpi_ext}
%define pkg_bindir %{pkg_prefix}/bin/
%define pkg_libdir %{pkg_prefix}/%{_lib}/
%define pkg_incdir %{pkg_prefix}/include/
%define pkg_datadir %{pkg_prefix}/share/
%define pkg_sysconfdir %{pkg_prefix}/etc/
%define pkg_skeldir %{pkg_prefix}/etc/skel/
%define package_name   %{pname}%{?pkg_suffix}
%endif

%define FP16_version             4dfe081
%define FXdiv_version            b408327
%define XNNPACK_version          51a0103
%define cpphttplib_version       3b6597b
%define cpuinfo_version          1e83a2f
%define flatbuffers              01834de
%define fmt_version              1239137
%define gemmlowp_version         3fb5c17
%define gloo_version             5354032
%define kineto                   a054a4b
%define onnx_version             b8baa84
%define opentelemetrycpp_version a799f4a
%define pocketfft                9d3ab05
%define psimd_version            072586a
%define pthreadpool_version      4fe0e1e
%define pybind11_version         a2e59f0
%define sleef_version            56e1f79
%define tensorpipe               52791a2
# nested vendored
%define libnop 910b558
%define kleidiai                 cca02c2

%{?sle15_python_module_pythons}
Name:           python-torch%{?pkg_suffix}
Version:        2.7.1
Release:        2.1
Summary:        Deep learning framework aka pytorch/Caffe2
License:        Apache-2.0 AND BSD-2-Clause AND BSD-3-Clause AND MIT AND Zlib AND BSL-1.0
Group:          Development/Languages/Python
URL:            https://pytorch.org
Source0:        https://github.com/pytorch/pytorch/archive/v%{version}.tar.gz#/%{srcname}-%{version}.tar.gz
Source1:        releases.html
# License10: BSD-3-Clause
Source10:       https://github.com/facebookincubator/gloo/archive/%{gloo_version}.tar.gz#/gloo-%{gloo_version}.tar.gz
Source32:       gloo-gcc15-fix.patch
# License11: MIT
Source11:       https://github.com/yhirose/cpp-httplib/archive/%{cpphttplib_version}.tar.gz#/cpp-httplib-%{cpphttplib_version}.tar.gz
# License12: BSD-2-Clause
Source12:       https://github.com/pytorch/cpuinfo/archive/%{cpuinfo_version}.tar.gz#/cpuinfo-%{cpuinfo_version}.tar.gz
# License13: BSL-1.0
Source13:       https://github.com/zdevito/sleef/archive/%{sleef_version}.tar.gz#/sleef-%{sleef_version}.tar.gz
# License14: BSD-3-Clause
Source14:       https://github.com/pybind/pybind11/archive/%{pybind11_version}.tar.gz#/pybind11-%{pybind11_version}.tar.gz
# License15: MIT
Source15:       https://github.com/onnx/onnx/archive/%{onnx_version}.tar.gz#/onnx-%{onnx_version}.tar.gz
#License16: BSD-2-Clause
Source16:       https://github.com/Maratyszcza/pthreadpool/archive/%{pthreadpool_version}.tar.gz#/pthreadpool-%{pthreadpool_version}.tar.gz
# License17: MIT
Source17:       https://github.com/Maratyszcza/FXdiv/archive/%{FXdiv_version}.tar.gz#/FXdiv-%{FXdiv_version}.tar.gz
# License18: MIT
Source18:       https://github.com/Maratyszcza/psimd/archive/%{psimd_version}.tar.gz#/psimd-%{psimd_version}.tar.gz
# License19: MIT
Source19:       https://github.com/Maratyszcza/FP16/archive/%{FP16_version}.tar.gz#/FP16-%{FP16_version}.tar.gz
# License20: Apache-2.0
Source20:       https://github.com/google/gemmlowp/archive/%{gemmlowp_version}.tar.gz#/gemmlowp-%{gemmlowp_version}.tar.gz
# License22: Apache-2.0
Source22:       https://github.com/open-telemetry/opentelemetry-cpp/archive/%{opentelemetrycpp_version}.tar.gz#/opentelemetry-cpp-%{opentelemetrycpp_version}.tar.gz
# License23: BSD-3-Clause
Source23:       https://github.com/google/XNNPACK/archive/%{XNNPACK_version}.tar.gz#/XNNPACK-%{XNNPACK_version}.tar.gz
# License 25: MIT
Source25:       https://github.com/fmtlib/fmt/archive/%{fmt_version}.tar.gz#/fmt-%{fmt_version}.tar.gz
# License 26: BSD-3-Clause
Source26:       https://github.com/mreineck/pocketfft/archive/%{pocketfft}.tar.gz#/pocketfft-%{pocketfft}.tar.gz
# License 27: BSD-3-Clause
Source27:       https://github.com/pytorch/kineto/archive/%{kineto}.tar.gz#/kineto-%{kineto}.tar.gz
# License 28: Apache-2.0
Source28:       https://github.com/google/flatbuffers/archive/%{flatbuffers}.tar.gz#/flatbuffers-%{flatbuffers}.tar.gz
# License 29: BSD-3-Clause
Source29:       https://github.com/pytorch/tensorpipe/archive/%{tensorpipe}.tar.gz#/tensorpipe-%{tensorpipe}.tar.gz
# License 30: Apache-2.0
Source30:       https://github.com/google/libnop/archive/%{libnop}.tar.gz#/libnop-%{libnop}.tar.gz
# yes this is patch to vendored file and upstream has archived this one, so this will stay
# adding it as a source avoid issues with autosetup
Source31:       add-cstdint.patch
# License 33: Apache-2.0 and BSD-3-Clause
Source33:       https://github.com/ARM-software/kleidiai/archive/%{kleidiai}.tar.gz#/kleidiai-%{kleidiai}.tar.gz


Patch1:         skip-third-party-check.patch
Patch2:         fix-setup.patch
Patch3:         pytorch-patch-onednn.patch

# A python call to cmake fails with a return code of 1 on this arch, disable it for now.
# and 32-bit arm is not supported
ExcludeArch:    %ix86 %{arm}

BuildRequires:  %{python_module Gloo}
%ifarch x86_64
BuildRequires:  %{python_module PeachPy}
%endif
BuildRequires:  %{python_module PyYAML}
BuildRequires:  %{python_module devel}
BuildRequires:  %{python_module hypothesis}
BuildRequires:  %{python_module numpy-devel}
BuildRequires:  %{python_module opcodes}
BuildRequires:  %{python_module pip}
BuildRequires:  %{python_module protobuf}
BuildRequires:  %{python_module psutil}
BuildRequires:  %{python_module py-cpuinfo}
BuildRequires:  %{python_module setuptools}
BuildRequires:  %{python_module typing_extensions}
BuildRequires:  %{python_module typing}
# Python 3.6 still need dataclasses
BuildRequires:  %{python_module dataclasses if %python-base < 3.7}
BuildRequires:  cmake >= 3.5
BuildRequires:  eigen3-devel
BuildRequires:  fdupes
%if 0%{?suse_version} >= 1600
BuildRequires:  gcc-c++
%else
BuildRequires:  gcc12
BuildRequires:  gcc12-c++
%endif
BuildRequires:  git
BuildRequires:  glog-devel
BuildRequires:  gtest
BuildRequires:  leveldb-devel
BuildRequires:  libnuma-devel
BuildRequires:  libopenblas_pthreads-devel
BuildRequires:  libuv-devel
BuildRequires:  lmdb-devel
BuildRequires:  ninja
BuildRequires:  nlohmann_json-devel
%ifarch x86_64 aarch64 ppc64le
BuildRequires:  onednn-devel
%endif
BuildRequires:  openblas-devel
BuildRequires:  opencl-cpp-headers
BuildRequires:  opencl-headers
BuildRequires:  opencv-devel
BuildRequires:  openssl-devel
BuildRequires:  protobuf-c
BuildRequires:  protobuf-devel
BuildRequires:  python-rpm-macros
BuildRequires:  snappy-devel
%if %{with cuda}
BuildRequires:  cuda-compiler-%cudaver
BuildRequires:  cuda-cudart-dev-%cudaver
BuildRequires:  cuda-libraries-dev-%cudaver
BuildRequires:  cuda-misc-headers-%cudaver
BuildRequires:  cuda-nsight-%cudaver
BuildRequires:  cuda-toolkit-%cudaver
BuildRequires:  libcudnn7-devel
BuildRequires:  libnccl-devel
%endif
%if %{with openmpi4}
BuildRequires:  openmpi4-devel
Conflicts:      %{python_module torch}
%endif
%if %{with vulkan}
BuildRequires:  VulkanMemoryAllocator-devel
BuildRequires:  shaderc
BuildRequires:  vulkan-devel
Conflicts:      %{python_module torch}
%endif
Requires:       python-numpy
Requires:       python-protobuf
Requires:       python-six
Requires:       python-sympy
Requires:       python-typing_extensions

Provides:       python-caffe2%{?pkg_suffix} = %version
Provides:       python-pytorch%{?pkg_suffix} = %version

%if "%flavor" == ""
ExclusiveArch:  do_not_build
%endif

%python_subpackages

%description
PyTorch enables fast, flexible experimentation and efficient production through
a hybrid front-end, distributed training, and ecosystem of tools and libraries.
The library is developed by Facebook and other groups.
PyTorch provides two high-level features:
* Tensor computing (like NumPy) with strong acceleration via graphics
* processing units (GPU) Deep neural networks built on a tape-based autodiff
  system

%package devel
Summary:        Headers for C/C++, cmake build description and libraries needed for development
Group:          Development/Languages/Python
Requires:       python-torch = %{version}
%if %{with openmpi4}
Conflicts:      %{python_module torch-devel}
%endif
%if %{with vulkan}
Conflicts:      %{python_module torch-devel}
%endif

%description devel
Although the Python interface is more polished and the primary focus of
development, PyTorch also has a C++ frontend. This package contains the header
to access the C/C++ interface.

%package converters
Summary:        Converters for onnx and caffe2
Group:          Development/Languages/Python
BuildArch:      noarch
Requires:       python3-click
Requires:       python3-onnx
Requires:       python3-pip
Requires:       python3-pname
%if %{with openmpi4}
Conflicts:      %{python_module torch-converters}
%endif
%if %{with vulkan}
Conflicts:      %{python_module torch-converters}
%endif

%description converters
Converter from caffe2 to onnx and from caffe2 to onnx formated files.

%package -n libtorch%{?pkg_suffix}
Summary:        Library which used by %{name}
Group:          Development/Libraries/Python
%if %{with openmpi4}
Conflicts:      libtorch
%endif
%if %{with vulkan}
Conflicts:      libtorch
%endif

%description -n libtorch%{?pkg_suffix}
Library which is used by %{name}

%prep
%define make_depend_src() test -e $(basename %1| sed 's/-.*//') && rmdir %{?2}%{!?2:$(basename %1| sed 's/-.*//')}; tar xzf %1; mv $(basename %{1} | sed 's/\.tar\.gz//' )* %{?2}%{!?2:$(basename %1| sed 's/-.*//')}
%define make_depend_src_uppercase() rmdir -p $(basename %1| sed 's/-.*//'| tr '[:upper:]' '[:lower:]'); tar xzf %1; mv $(basename %1 | cut -f 1 -d '.' ) $(basename %1| sed 's/-.*//'| tr '[:upper:]' '[:lower:]')
%autosetup -p1 -n %{srcname}-%{version}
cp %{S:1} releases.html
%if %{with vulkan}
sed -i '/-Werror=return-type/d'  CMakeLists.txt
%endif
cd third_party
rmdir python-peachpy/
rmdir eigen/
%make_depend_src %{SOURCE10}
patch -d ./gloo/ -p1 < %{SOURCE32}
%make_depend_src %{SOURCE11}
rmdir cpp-httplib && mv cpp cpp-httplib
%make_depend_src %{SOURCE12}
%make_depend_src %{SOURCE13}
%make_depend_src %{SOURCE14}
%make_depend_src %{SOURCE15}
%make_depend_src %{SOURCE16}
%make_depend_src %{SOURCE17}
%make_depend_src %{SOURCE18}
%make_depend_src %{SOURCE19}
%make_depend_src %{SOURCE20} gemmlowp/gemmlowp
%make_depend_src %{SOURCE22}
rmdir opentelemetry-cpp && mv opentelemetry opentelemetry-cpp
%make_depend_src %{SOURCE23}
%make_depend_src %{SOURCE25}
%make_depend_src %{SOURCE26}
%make_depend_src %{SOURCE27}
%make_depend_src %{SOURCE28}
%make_depend_src %{SOURCE29}
%make_depend_src %{SOURCE33}
# getting the vendoring of the vendored source working, this is
# insanity at the next level. My only exuse is that libnop is header only
rmdir tensorpipe/third_party/libnop
%make_depend_src %{SOURCE30} tensorpipe/third_party/libnop
# and a patch is needed
patch -d ./tensorpipe/ -p1 < %{SOURCE31}
cd -

# Fix installation path
sed -i -e 's@DESTINATION ${PYTHON_LIB_REL_PATH}@DESTINATION ${CMAKE_INSTALL_PREFIX}/${PYTHON_LIB_REL_PATH}@' caffe2/CMakeLists.txt
# Disable nccl download
sed -i -e '/[\s]*checkout_nccl()/d' tools/build_pytorch_libs.py

%build
cat > buildvars <<EOF
export USE_KLEIDIAI=ON
export USE_SYSTEM_LIBS=OFF
export USE_NNPACK=OFF
%if %{with cuda}
export USE_SYSTEM_NCCL=ON
export NCCL_INCLUDE_DIR="/usr/include/"
export USE_CUDA=ON
export USE_CUDNN=ON
export PATH="/usr/local/cuda-10.1/bin:$PATH"
export CPLUS_INCLUDE_PATH="/usr/local/cuda-10.1/include"
export C_INCLUDE_PATH="/usr/local/cuda-10.1/include"
export LD_LIBRARY_PATH="/usr/local/cuda-10.1/lib"
%else
export USE_CUDA=OFF
export USE_CUDNN=OFF
%endif
%if 0%{?suse_version} >= 1600
export CC=gcc-%gcc_version
export CXX=g++-%gcc_version
%else
export CC=gcc-12
export CXX=g++-12
%endif
export USE_KINETO=OFF
export USE_FBGEMM=OFF
export USE_SYSTEM_BENCHMARK=ON
export USE_SYSTEM_EIGEN_INSTALL=ON
%if 0%{?sle_version} == 150600
export USE_MKLDNN=OFF
export USE_OPENCL=OFF
%else
export USE_MKLDNN=ON
%endif
export USE_DISTRIBUTED=ON
export TP_BUILD_LIBUV=OFF
export USE_NCCL=OFF
export USE_OPENMP=ON
export USE_LAPACK=ON
%if %{with mpi}
export USE_MPI=ON
export MPIEXEC_EXECUTABLE="%{pkg_bindir}/mpiexec"
%else
export USE_MPI=OFF
export USE_GLOO=ON
%endif
export BLAS=OpenBLAS
export BUILD_CUSTOM_PROTOBUF=OFF
export BUILD_TEST=OFF
export MAX_JOBS=%{?jobs}
%if %{with vulkan}
export USE_VULKAN=ON
export CXXFLAGS="-I /usr/ -Wno-error=return-type"
%endif
%ifarch aarch64
export CFLAGS="$CFLAGS -Wno-incompatible-pointer-types"
export CXXFLAGS="$CXXFLAGS -Wno-incompatible-pointer-types"
%endif
EOF

source ./buildvars

%python_build

%install

source ./buildvars

%python_install -q

%python_expand %fdupes %{buildroot}%{$python_sitearch}

install -m 644 -D %{buildroot}%{python_sitearch}/torch/lib/* %{buildroot}/%{_libdir}
#rm -r %{buildroot}%{python_sitearch}/torch/lib
#cd %{buildroot}/%{_libdir}
#rm libtorch.so
#ln -s libtorch.so.1 libtorch.so
#cd -
#for file in  $(find %{buildroot}%{python_sitearch} -type f -name \*.py -perm 644 -size +1b); do
#%{__grep} '/usr/bin/env ' $file && sed -i 's@/usr/bin/env python@/usr/bin/python@' $file && chmod 755 $file
#done
#
#%check
#export LD_LIBRARY_PATH=%{buildroot}/%{_libdir}
#%%python_expand PYTHONPATH=%{buildroot}%{$python_sitearch} $python test/run_test.py
%python_clone -a %{buildroot}%{_bindir}/torchfrtrace
%python_clone -a %{buildroot}%{_bindir}/torchrun

%post -n libtorch%{?pkg_suffix} -p /sbin/ldconfig
%postun -n libtorch%{?pkg_suffix} -p /sbin/ldconfig

%files %{python_files}
%defattr(-,root,root)
%doc README.md NOTICE releases.html
%license LICENSE
%{python_sitearch}/torch*
%{python_sitearch}/torchgen/
%{python_sitearch}/functorch/
%exclude %{python_sitearch}/torch/share
%exclude %{python_sitearch}/torch/include
%exclude %{python_sitearch}/torch/_inductor/codegen
%exclude %{python_sitearch}/torch/utils/benchmark/utils/
%exclude %{python_sitearch}/torchgen/packaged/ATen/templates
%exclude %{python_sitearch}/torchgen/packaged/autograd/templates
%python_alternative %{_bindir}/torchfrtrace
%python_alternative %{_bindir}/torchrun

%files %{python_files devel}
%{python_sitearch}/torch/share
%{python_sitearch}/torch/include
%{python_sitearch}/torch/_inductor/codegen
%{python_sitearch}/torch/utils/benchmark/utils/
%{python_sitearch}/torchgen/packaged/ATen/templates
%{python_sitearch}/torchgen/packaged/autograd/templates

%files -n libtorch%{?pkg_suffix}
%{_libdir}/*.so*

%changelog
* Thu Jul 10 2025 Guillaume GARDET <guillaume.gardet@opensuse.org>
- Add kleidiai dep (vendored)
- Workaround build error on aarch64 with -Wno-incompatible-pointer-types
* Fri Jul  4 2025 Christian Goll <cgoll@suse.com>
- Updated to 2.7.1 with following fiex:
  * Fix assertion error due to inductor permuting inputs to flex attention (#151959)
  * Fix performance regression on nanogpt speedrun (#152641)
  * Fix improve PyTorch Wheel size due to introduction of addition of 128 bit vectorization (#148320) (#152396)
  * Fix fmsub function definition (#152075)
  * Fix Floating point exception in torch.mkldnn_max_pool2d (#151848)
  * Fix ONNX decomposition does not preserve custom CompositeImplicitAutograd ops (#151826)
  * Fix error with dynamic linking of libgomp library (#150084)
  * Fix segfault in profiler with Python 3.13 (#153848)
- Changes from 2.7.0:
  * torch.onnx.dynamo_export now uses the ExportedProgram logic path (#137296)
    Users using the torch.onnx.dynamo_export API may see some ExportOptions
    become unsupported due to an internal switch to use torch.onnx.export(...,
    dynamo=True): diagnostic_options, fake_context and onnx_registry are
    removed/ignored by ExportOptions. Only dynamic_shapes is retained.
  * Finish deprecation of LRScheduler.print_lr() along with the verbose kwarg
    to the LRScheduler constructor. (#147301)
  * libtorch_python.so symbols are now invisible by default on all platforms (#142214)
  * Please use torch.export.export instead of capture_pre_autograd_graph to
    export the model for pytorch 2 export quantization (#139505)
  * New interface for
    torch.fx.passes.graph_transform_observer.GraphTransformObserver to enable
    Node Level provenance tracking (#144277)
  * torch.ao.quantization.pt2e.graph_utils.get_control_flow_submodules is no
    longer public (#141612)
  * torch.onnx.dynamo_export is deprecated (#146425, #146639, #146923)
  * XNNPACKQuantizer is deprecated in PyTorch and moved to ExecuTorch, please
    use it from executorch.backends.xnnpack.quantizer.xnnpack_quantizer instead
    of torch.ao.quantization.quantizer.xnnpack_quantizer. (#144940)
- Changes from 2.6.0
  * [Beta] torch.compiler.set_stance
  * [Beta] torch.library.triton_op
  * [Beta] torch.compile support for Python 3.13
  * [Beta] New packaging APIs for AOTInductor
  * [Beta] New packaging APIs for AOTInductor
  * [Beta] AOTInductor: minifier
  * [Beta] AOTInductor: ABI-compatible mode code generation
  * [Beta] FP16 support for X86 CPUs (both eager and Inductor modes)
  * FlexAttention support on X86 CPU for LLMs
- Added patches for gcc15 compatibilty:
  * add-cstdint.patch
  * gloo-gcc15-fix.patch
- Updated vendored sources:
  * kineto-d975313.tar.gz -> kineto-a054a4b.tar.gz
  * onnx-3bf92c0.tar.gz -> onnx-b8baa84.tar.gz
  * pybind11-7c33cdc.tar.gz -> pybind11-a2e59f0.tar.gz
  * sleef-60e76d2.tar.gz -> sleef-56e1f79.tar.gz
  * XNNPACK-fcbf55a.tar.gz -> XNNPACK-51a0103.tar.gz
  * cpuinfo-fa1c679.tar.gz -> cpuinfo-1e83a2f.tar.gz
  * fmt-e69e5f9.tar.gz -> fmt-1239137.tar.gz
* Tue Dec 17 2024 Andreas Schwab <schwab@suse.de>
- Use oneDNN only on x86_64 aarch64 ppc64le
* Fri Oct 18 2024 Guillaume GARDET <guillaume.gardet@opensuse.org>
-  Update to 2.5.0:
  * https://github.com/pytorch/pytorch/releases/tag/v2.5.0
* Fri Oct  4 2024 Guillaume GARDET <guillaume.gardet@opensuse.org>
- Add patch to fix build with oneDNN:
  * pytorch-patch-onednn.patch
* Tue Oct  1 2024 Guillaume GARDET <guillaume.gardet@opensuse.org>
- Update to 2.4.1:
  * https://github.com/pytorch/pytorch/releases/tag/v2.4.1
- Skip update to 2.4.0:
  * https://github.com/pytorch/pytorch/releases/tag/v2.4.0
- Remove _service since 'osc mr download_files' is easier to use
  and maintain
- Drop config vars not used anymore: BUILD_CAFFE2, USE_LEVELDB, USE_LMDB,
  USE_OPENCV, USE_TBB
- Remove examples package since code has been removed upstream
- Refresh pacth:
  * skip-third-party-check.patch
* Thu Aug 29 2024 Guang Yee <gyee@suse.com>
- Enable sle15_python_module_pythons.
- GCC 9.3 or newer is required, regardless if CUDA is enabled.
  See https://github.com/pytorch/pytorch/blob/v2.3.1/CMakeLists.txt#L48
  Therefore, for SLE15 we went with GCC 11 as it seems to be the most
  common one.
- Use %%gcc_version macro for Tumbleweed.
* Thu Jul 11 2024 Christian Goll <cgoll@suse.com>
- update to 2.3.1 with following summarized highlights:
  * from 2.0.x:
  - torch.compile is the main API for PyTorch 2.0, which wraps your model and
    returns a compiled model. It is a fully additive (and optional) feature
    and hence 2.0 is 100%% backward compatible by definition
  - Accelerated Transformers introduce high-performance support for training
    and inference using a custom kernel architecture for scaled dot product
    attention (SPDA). The API is integrated with torch.compile() and model
    developers may also use the scaled dot product attention kernels directly
    by calling the new scaled_dot_product_attention() operato
  * from 2.1.x:
  - automatic dynamic shape support in torch.compile,
    torch.distributed.checkpoint for saving/loading distributed training jobs
    on multiple ranks in parallel, and torch.compile support for the NumPy
    API.
  - In addition, this release offers numerous performance improvements (e.g.
    CPU inductor improvements, AVX512 support, scaled-dot-product-attention
    support) as well as a prototype release of torch.export, a sound
    full-graph capture mechanism, and torch.export-based quantization.
  * from 2.2.x:
  - 2x performance improvements to scaled_dot_product_attention via
    FlashAttention-v2 integration, as well as AOTInductor, a new
    ahead-of-time compilation and deployment tool built for non-python
    server-side deployments.
  * from 2.3.x:
  - support for user-defined Triton kernels in torch.compile, allowing for
    users to migrate their own Triton kernels from eager without
    experiencing performance complications or graph breaks. As well, Tensor
    Parallelism improves the experience for training Large Language Models
    using native PyTorch functions, which has been validated on training
    runs for 100B parameter models.
- added seperate openmpi4 build
- added sepetate vulcan build, although this functions isn't exposed to python
  abi
- For the obs build all the vendored sources follow the pattern
  NAME-7digitcommit.tar.gz and not the NAME-COMMIT.tar.gz
- added following patches:
  * skip-third-party-check.patch
  * fix-setup.patch
- removed patches:
  * pytorch-rm-some-gitmodules.patch
  * fix-call-of-onnxInitGraph.patch
* Thu Jul 22 2021 Guillaume GARDET <guillaume.gardet@opensuse.org>
- Fix build on x86_64 by using GCC10 instead of GCC11
  https://github.com/google/XNNPACK/issues/1550
* Thu Jul 22 2021 Guillaume GARDET <guillaume.gardet@opensuse.org>
- Update to 1.9.0
- Release notes: https://github.com/pytorch/pytorch/releases/tag/v1.9.0
- Drop upstreamed patch:
  * fix-mov-operand-for-gcc.patch
- Drop unneeded patches:
  * removed-peachpy-depedency.patch
- Refresh patches:
  * skip-third-party-check.patch
  * fix-call-of-onnxInitGraph.patch
- Add new patch:
  * pytorch-rm-some-gitmodules.patch
* Thu Jul 22 2021 Guillaume GARDET <guillaume.gardet@opensuse.org>
- Add _service file to ease future update of deps
* Thu Jul 22 2021 Guillaume GARDET <guillaume.gardet@opensuse.org>
- Update sleef to fix build on aarch64
* Fri Apr 23 2021 Matej Cepl <mcepl@suse.com>
- Don't build python36-* package (missing pandas)
* Thu Jan 21 2021 Benjamin Greiner <code@bnavigator.de>
- Fix python-rpm-macros usage
* Wed Oct  7 2020 Guillaume GARDET <guillaume.gardet@opensuse.org>
- Use GCC9 to build on aarch64 Tumbleweed to workaround SVE
  problem with GCC10 with sleef, see:
  https://github.com/pytorch/pytorch/issues/45971
* Thu Aug 20 2020 Martin Liška <mliska@suse.cz>
- Use memoryperjob constraint instead of %%limit_build macro.
* Tue Jun 23 2020 Christian Goll <cgoll@suse.com>
- updated to new stable release 1.5.1 which has following changes:
  This release includes several major new API additions and improvements. These
  include new APIs for autograd allowing for easy computation of hessians and
  jacobians, a significant update to the C++ frontend, ‘channels last’ memory
  format for more performant computer vision models, a stable release of the
  distributed RPC framework used for model parallel training, and a new API
  that allows for the creation of Custom C++ Classes that was inspired by
  PyBind. Additionally torch_xla 1.5 is now available and tested with the
  PyTorch 1.5 release providing a mature Cloud TPU experience.
  * see release.html for detailed information
- added patches:
  * fix-call-of-onnxInitGraph.patch for API mismatch in onnx
  * fix-mov-operand-for-gcc.patch for aarch64 operands
- removed sources:
  * cpuinfo-89fe1695edf9ee14c22f815f24bac45577a4f135.tar.gz
  * gloo-7c541247a6fa49e5938e304ab93b6da661823d0f.tar.gz
  * onnx-fea8568cac61a482ed208748fdc0e1a8e47f62f5.tar.gz
  * psimd-90a938f30ba414ada2f4b00674ee9631d7d85e19.tar.gz
  * pthreadpool-13da0b4c21d17f94150713366420baaf1b5a46f4.tar.gz
- added sources:
  * cpuinfo-0e6bde92b343c5fbcfe34ecd41abf9515d54b4a7.tar.gz
  * gloo-113bde13035594cafdca247be953610b53026553.tar.gz
  * onnx-9fdae4c68960a2d44cd1cc871c74a6a9d469fa1f.tar.gz
  * psimd-10b4ffc6ea9e2e11668f86969586f88bc82aaefa.tar.gz
  * pthreadpool-d465747660ecf9ebbaddf8c3db37e4a13d0c9103.tar.gz
* Tue Jun 23 2020 Christian Goll <cgoll@suse.com>
- updated to bugfix release 1.4.1 and added _multibuild file so
  that cuda versions can be build on commandline
* Thu Apr 23 2020 Tomáš Chvátal <tchvatal@suse.com>
- Make sure to pull py2/py3 package from the devel pkg
* Thu Apr 23 2020 Tomáš Chvátal <tchvatal@suse.com>
- Do not pull in python2 only dependencies
* Wed Feb 26 2020 Simon Lees <sflees@suse.de>
- Exclude i586 builds for now, they fail with a cryptic return
  code of 1 from cmake from python.
* Fri Feb 21 2020 Christian Goll <cgoll@suse.com>
- updated to stable release 1.4.0, which has as Highlights:
  * Distributed Model Parallel Training
  * Pruning functionalities have been added to PyTorch
- New Features:
  * torch.optim.lr_scheduler now support “chaining.”
  * torch.distributed.rpc is a newly introduced package
- full Changelog listed in relases file or under
  https://github.com/pytorch/pytorch/releases
  and in the releases.hml file
- added files:
  * skip-third-party-check.patch which is a patch to skip
    the check of disabled dependencies
  * QNNPACK-7d2a4e9931a82adc3814275b6219a03e24e36b4c.tar.gz
    which is part of pytorch but developed in different repo
  * releases.html which is the downloaded releases file
- removed patch files:
  * fix-build-options.patch
  * honor-PSIMD-env.patch
  * removed-some-tests.patch
* Tue Jan 14 2020 Guillaume GARDET <guillaume.gardet@opensuse.org>
- Requires python-PeachPy on x86_64 only, as it is optional
  and available on x86_64 only
* Wed Jan  8 2020 Christian Goll <cgoll@suse.com>
- updated the requirement for examples and converters
* Wed Jun 12 2019 Christian Goll <cgoll@suse.com>
- Updated to stable version 1.1.0, which needed also updates of
  following dependend sources:
  * onnx-1.4.1.tar.gz ->
    onnx-22662bfd4dcc6baebf29e3b823a051676f991001.tar.gz
- Removed following sources:
  * FBGEMM-f65f0ebe54f0512d8f42ee10025b596e3f42e0b8.tar.gz
- Added following sources:
  * foxi-8f74bc4df3a4cfc69b1a3eadf62aa29d9961c72d.tar.gz
- Changed patch
  * fix-build-options.patch to work with new buid system and
    exclude FBGEMM
- Added patch:
  * honor-PSIMD-env.patch, which makes depend sources of pytorch
    to use the source of psimd
* Tue Mar 26 2019 Christian Goll <cgoll@suse.com>
- Inital commit of pytorch/caffe2 which is an opensource
  machineleraning platform. This is the stable release 1.0.1
  including like other tools a lot of third party sources,
  which could not be used from the base system due to messy
  build system. Additional sources are
  * gloo, a communitcation library for GPUs as
    gloo-670b4d4aa46886cc66874e2a4dc846f5cfc2a285.tar.gz
  * fbgemm, a low precission, high peformance matrix lib
    FBGEMM-f65f0ebe54f0512d8f42ee10025b596e3f42e0b8.tar.gz
  * cpuinfo, a cross platform cpu information tool
    cpuinfo-89fe1695edf9ee14c22f815f24bac45577a4f135.tar.gz
  * sleef, a function for elementary functions
    sleef-191f655caa25526ae226cf88dd2529265176014a.tar.gz
  * pytbind11, which exposes C/C++ headers to pythob, but
    the source code of this library is deeply integrated into
    pytorch, so we need
    pybind11-25abf7efba0b2990f5a6dfb0a31bc65c0f2f4d17.tar.gz
  * onnx, which is an format for exchaning neural networks as
    onnx-1.4.1.tar.gz
  * pthreadpool, a pthread based thread tool implementation, which
    can be used when omp is not available
    pthreadpool-13da0b4c21d17f94150713366420baaf1b5a46f4.tar.gz
  * FXdiv, a Header-only library for division via fixed-point
    multiplication by inverse, which has no stable API atm, so
    FXdiv-b742d1143724d646cd0f914646f1240eacf5bd73.tar.gz
  * psimd, portable 128-bit SIMD intrinsics
    psimd-90a938f30ba414ada2f4b00674ee9631d7d85e19.tar.gz
  * fp16, a numeric conversion library
    FP16-febbb1c163726b5db24bed55cc9dc42529068997.tar.gz
  * gemmlowp, self-contained low-precision GEMM library as
    gemmlowp-8416bab644641a5c0a81ecf91a5cda804af0aee1.tar.gz
  * fix-build-options.patch, which points pytorch to system libs
  * removed-peachpy-depedency.patch, which forces to use system
    peachpy
